package com.qmth.teachcloud.report.business.service;

import org.springframework.scheduling.annotation.Async;

/**
 * @Description: 变更名称服务类（）
 * @Author: CaoZixuan
 * @Date: 2022-03-10
 */
public interface ChangNamesService {

    /**
     * 更新基础表数据
     */
    @Async("analyzeThreadPool")
    void changeBasicTables();

    /**
     * 更新表't_a_exam_course'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourse() throws InterruptedException;

    /**
     * 更新表 't_a_exam_course_clazz'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseClazz();

    /**
     * 更新表 't_a_exam_course_college_inspect'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseCollegeInspect();

    /**
     * 更新表 't_a_exam_course_college_inspect_dio'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseCollegeInspectDio();

    /**
     * 更新表 't_a_exam_course_college_paper_struct'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseCollegePaperStruct();

    /**
     * 更新表 't_a_exam_course_college_teacher'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseCollegeTeacher();

    /**
     * 更新表 't_a_exam_course_difficult'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseDifficultService();

    /**
     * 更新表 't_a_exam_course_dio'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseDio();

    /**
     * 更新表 't_a_exam_course_record_dio'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseRecordDio();

    /**
     * 更新表 't_a_exam_course_record_mod'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseRecordMod();

    /**
     * 更新表 't_a_exam_course_teacher'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacher();

    /**
     * 更新表 't_a_exam_course_teacher_college_difficult'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacherCollegeDifficult();

    /**
     * 更新表 't_a_exam_course_teacher_college_dio'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacherCollegeDio();

    /**
     * 更新表 't_a_exam_course_teacher_college_paper_struct'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacherCollegePaperStruct();

    /**
     * 更新表 't_a_exam_course_teacher_difficult'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacherDifficult();

    /**
     * 更新表 't_a_exam_course_teacher_dio'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacherDio();

    /**
     * 更新表 't_a_exam_course_teacher_paper_struct'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamCourseTeacherPaperStruct();

    /**
     * 更新表 't_a_exam_total'
     */
    @Async("analyzeThreadPool")
    void changeTableTAExamTotal();

    /**
     * 更新表 't_a_paper_struct'
     */
    @Async("analyzeThreadPool")
    void changeTableTAPaperStruct();
}
